@@ -5,7 +5,7 @@ require_once KRONOLITH_BASE . '/lib/Day.php';
 /**
  * The Kronolith_DayView:: class provides an API for viewing days.
  *
- * $Horde: kronolith/lib/DayView.php,v 1.139.2.8 2005/10/20 09:17:49 jan Exp $
+ * $Horde: kronolith/lib/DayView.php,v 1.139.2.12 2006/10/24 09:16:37 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
@@ -44,7 +44,7 @@ class Kronolith_DayView extends Kronolith_Day {
                                             new Horde_Date(array('year' => $this->year, 'month' => $this->month, 'mday' => $this->mday,
                                                                  'hour' => 23, 'min' => 59, 'sec' => 59)),
                                             $GLOBALS['display_calendars']);
-            $this->_events = array_pop($events);
+            $this->_events = array_shift($events);
         } else {
             $this->_events = $events;
         }
@@ -69,6 +69,10 @@ class Kronolith_DayView extends Kronolith_Day {
 
         $started = false;
         $first_row = true;
+        $addLinks = Kronolith::getDefaultCalendar(PERMS_EDIT) && !$print_view &&
+            (!empty($GLOBALS['conf']['hooks']['permsdenied']) ||
+             Kronolith::hasPermission('max_events') === true ||
+             Kronolith::hasPermission('max_events') > Kronolith::countEvents());
 
         require $template_path . '/day/head.inc';
         if ($this->_sidebyside) {
@@ -79,14 +83,10 @@ class Kronolith_DayView extends Kronolith_Day {
         $eventCategories = array();
 
         $row = '';
-        if (!$print_view &&
-            Kronolith::getDefaultCalendar(PERMS_EDIT) &&
-            (!empty($conf['hooks']['permsdenied']) ||
-             Kronolith::hasPermission('max_events') === true ||
-             Kronolith::hasPermission('max_events') > Kronolith::countEvents())) {
+        if ($addLinks) {
             $addeventurl = Util::addParameter('addevent.php', array('timestamp' => $this->slots[0]['timestamp'],
                                                                     'allday' => 1,
-                                                                    'url' => Horde::selfUrl(true)));
+                                                                    'url' => Horde::selfUrl(true, true, true)));
             $addeventurl = Horde::link(Horde::applicationUrl($addeventurl), _("Create a New Event"), 'hour') . _("All day") .
                 Horde::img('new_small.png', '+') . '</a>';
         } else {
@@ -114,6 +114,7 @@ class Kronolith_DayView extends Kronolith_Day {
                     $eventCategories[$event->getCategory()] = true;
 
                     $row .= '<td class="day-eventbox category' . md5($event->getCategory()) . '" ' .
+                        'width="' . round(90 / count($this->_currentCalendars))  . '%" ' .
                         'valign="top" colspan="' . $this->_span[$cid] . '">' .
                         $event->getLink($this->getStamp()) . '</td>';
                 }
@@ -184,14 +185,10 @@ class Kronolith_DayView extends Kronolith_Day {
                 }
             }
 
-            if (!$print_view &&
-                Kronolith::getDefaultCalendar(PERMS_EDIT) &&
-                (!empty($conf['hooks']['permsdenied']) ||
-                 Kronolith::hasPermission('max_events') === true ||
-                 Kronolith::hasPermission('max_events') > Kronolith::countEvents())) {
+            if ($addLinks) {
                 $addeventurl = Util::addParameter('addevent.php',
                                                   array('timestamp' => $this->slots[$i]['timestamp'],
-                                                        'url' => Horde::selfUrl(true)));
+                                                        'url' => Horde::selfUrl(true, true, true)));
                 $addeventurl = Horde::link(Horde::applicationUrl($addeventurl), _("Create a New Event"), $hourclass) .
                     $time . Horde::img('new_small.png', '+') . '</a>';
             } else {
@@ -252,15 +249,6 @@ class Kronolith_DayView extends Kronolith_Day {
                 // Initialize this event's vertical span.
                 $event->rowspan = 0;
 
-                // Make sure that we're using the current date for
-                // recurring events.
-                if (!$event->hasRecurType(KRONOLITH_RECUR_NONE)) {
-                    $event->start = &Util::cloneObject($event->start);
-                    $event->start->mday = $this->mday;
-                    $event->start->month = $this->month;
-                    $event->start->year = $this->year;
-                    $event->end = &new Horde_Date($event->start->timestamp() + $event->durMin * 60);
-                }
                 $tmp[] = &Util::cloneObject($event);
             }
         }
