@@ -3,7 +3,7 @@
  * Kronolith_Driver:: defines an API for implementing storage backends
  * for Kronolith.
  *
- * $Horde: kronolith/lib/Driver.php,v 1.116.2.43 2006/06/16 09:04:10 jan Exp $
+ * $Horde: kronolith/lib/Driver.php,v 1.116.2.54 2006/10/29 14:13:20 karsten Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Kronolith 0.1
@@ -240,11 +240,11 @@ class Kronolith_Event {
     var $title = '';
 
     /**
-     * The identifier of the category this event belongs to.
+     * The category of this event.
      *
-     * @var integer
+     * @var string
      */
-    var $category = 0;
+    var $category = '';
 
     /**
      * The location this event occurs at.
@@ -402,11 +402,10 @@ class Kronolith_Event {
      *
      * @param Horde_iCalendar &$calendar  A Horde_iCalendar object that acts as
      *                                    a container.
-     * @param Identity $identity          The Identity object of the organizer.
      *
      * @return Horde_iCalendar_vevent  The vEvent object for this event.
      */
-    function &toiCalendar(&$calendar, $identity)
+    function &toiCalendar(&$calendar)
     {
         global $prefs;
 
@@ -417,7 +416,7 @@ class Kronolith_Event {
 
         if ($this->isAllDay()) {
             $vEvent->setAttribute('DTSTART', $this->start, array('VALUE' => 'DATE'));
-            $vEvent->setAttribute('DTEND', $this->end, array('VALUE' => 'DATE'));
+            $vEvent->setAttribute('DTEND', new Horde_Date($this->end->timestamp()-1), array('VALUE' => 'DATE'));
         } else {
             $vEvent->setAttribute('DTSTART', $this->start);
             $vEvent->setAttribute('DTEND', $this->end);
@@ -427,10 +426,9 @@ class Kronolith_Event {
         $vEvent->setAttribute('UID', $this->_uid);
         $vEvent->setAttribute('SUMMARY', $this->title);
         $vEvent->setAttribute('TRANSP', 'OPAQUE');
-        $name = $identity->getValue('fullname');
         $vEvent->setAttribute('ORGANIZER',
-                              'MAILTO:' . $identity->getValue('from_addr'),
-                              empty($name) ? null : array('CN' => $name));
+                              'MAILTO:' . Kronolith::getUserEmail($this->getCreatorId()),
+                              array('CN' => Kronolith::getUserName($this->getCreatorId())));
         if (!empty($this->description)) {
             $vEvent->setAttribute('DESCRIPTION', $this->description);
         }
@@ -481,7 +479,8 @@ class Kronolith_Event {
             $vEvent->setAttribute('ATTENDEE', 'MAILTO:' . $email, $params);
         }
 
-        // Alarm.
+        // vCalendar 1.0 alarms. Has to be replaced with vAlarm components for
+        // RFC 2445, if anyone ever requests.
         if (!empty($this->alarm)) {
             $vEvent->setAttribute('AALARM', $this->start->timestamp() - $this->alarm * 60);
         }
@@ -589,6 +588,18 @@ class Kronolith_Event {
         if (!is_array($desc) && !is_a($desc, 'PEAR_Error')) {
             $this->setDescription($desc);
         }
+                /* All day events are transferred by many device as
+                 * DSTART: YYYYMMDDT000000 DTEND: YYYYMMDDT2359(59|00)
+                 * Convert accordingly */
+                if (is_object($this->start)
+                    && $this->start->hour == 0  && $this->start->min == 0
+                    && $this->start->sec == 0
+                    && $this->end->hour == 23 && $this->end->sec == 59) {
+                    $this->end = new Horde_Date(array(
+                                   'year' => (int)$this->end->year,
+                                   'month' => (int)$this->end->month,
+                                   'mday' => (int)$this->end->mday + 1));
+                }
 
         // Location.
         $location = $vEvent->getAttribute('LOCATION');
@@ -598,30 +609,39 @@ class Kronolith_Event {
 
         // Start and end date.
         $start = $vEvent->getAttribute('DTSTART');
-        if (!is_array($start) && !is_a($start, 'PEAR_Error')) {
-            $this->start = &new Horde_Date($start);
-        } elseif (is_array($start) && !is_a($start, 'PEAR_Error')) {
-            $this->start = &new Horde_Date(array('year' => (int)$start['year'],
-                                                 'month' => (int)$start['month'],
-                                                 'mday' => (int)$start['mday']));
+        if (!is_a($start, 'PEAR_Error')) {
+            if (!is_array($start)) {
+                // Date-Time field
+                $this->start = new Horde_Date($start);
+            } else {
+                // Date field
+                $this->start = new Horde_Date(array('year' => (int)$start['year'],
+                                                    'month' => (int)$start['month'],
+                                                    'mday' => (int)$start['mday']));
+            }
         }
         $end = $vEvent->getAttribute('DTEND');
-        if (!is_array($end) && !is_a($end, 'PEAR_Error')) {
-            $this->end = &new Horde_Date($end);
-        } elseif (is_array($end) && !is_a($end, 'PEAR_Error')) {
-            $this->end = &new Horde_Date(array('year' => (int)$end['year'],
-                                               'month' => (int)$end['month'],
-                                               'mday' => (int)$end['mday']));
+        if (!is_a($end, 'PEAR_Error')) {
+            if (!is_array($end)) {
+                // Date-Time field
+                $this->end = new Horde_Date($end);
+            } elseif (is_array($end) && !is_a($end, 'PEAR_Error')) {
+                // Date field
+                $this->end = new Horde_Date(array('year' => (int)$end['year'],
+                                                  'month' => (int)$end['month'],
+                                                  'mday' => (int)$end['mday'] + 1));
+            }
         } else {
             $duration = $vEvent->getAttribute('DURATION');
             if (!is_array($duration) && !is_a($duration, 'PEAR_Error')) {
-                $this->end = &new Horde_Date($this->start->timestamp() + $duration);
+                $this->end = new Horde_Date($this->start->timestamp() + $duration);
             } else {
                 // End date equal to start date as per RFC 2445.
                 $this->end = Util::cloneObject($this->start);
             }
         }
 
+        /* vCalendar 1.0 alarms. */
         $alarm = $vEvent->getAttribute('AALARM');
         if (!is_array($alarm) && !is_a($alarm, 'PEAR_Error') && intval($alarm)) {
             $this->alarm = intval(($this->start->timestamp() - $alarm) / 60);
@@ -833,16 +853,24 @@ class Kronolith_Event {
                 }
             }
             if (count($time) == 3 && count($date) == 3) {
-                $this->start = &new Horde_Date(array('year' => $date[0], 'month' => $date[1], 'mday' => $date[2],
-                                                     'hour' => $time[0], 'min' => $time[1], 'sec' => $time[2]));
+                $this->start = &new Horde_Date(array('year' => $date[0],
+                                                     'month' => $date[1],
+                                                     'mday' => $date[2],
+                                                     'hour' => $time[0],
+                                                     'min' => $time[1],
+                                                     'sec' => $time[2]));
             }
         } else {
             return PEAR::raiseError(_("Events must have a start date."));
         }
-        if (empty($hash['duration']) && empty($hash['end_date'])) {
-            $hash['end_date'] = $hash['start_date'];
-        }
-        if (!empty($hash['duration'])) {
+        if (empty($hash['duration'])) {
+            if (empty($hash['end_date'])) {
+                $hash['end_date'] = $hash['start_date'];
+            }
+            if (empty($hash['end_time'])) {
+                $hash['end_time'] = $hash['start_time'];
+            }
+        } else {
             $weeks = str_replace('W', '', $hash['duration'][1]);
             $days = str_replace('D', '', $hash['duration'][2]);
             $hours = str_replace('H', '', $hash['duration'][4]);
@@ -862,8 +890,12 @@ class Kronolith_Event {
                 }
             }
             if (count($time) == 3 && count($date) == 3) {
-                $this->end = &new Horde_Date(array('year' => $date[0], 'month' => $date[1], 'mday' => $date[2],
-                                                   'hour' => $time[0], 'min' => $time[1], 'sec' => $time[2]));
+                $this->end = &new Horde_Date(array('year' => $date[0],
+                                                   'month' => $date[1],
+                                                   'mday' => $date[2],
+                                                   'hour' => $time[0],
+                                                   'min' => $time[1],
+                                                   'sec' => $time[2]));
             }
         }
         if (!empty($hash['alarm'])) {
@@ -1178,10 +1210,9 @@ class Kronolith_Event {
     }
 
     /**
-     * Set the category this event belongs to.
+     * Set the category of this event.
      *
-     * @param integer $category  The identifier of the category this event
-     *                           belongs to.
+     * @param string $category  The category of this event.
      */
     function setCategory($category)
     {
@@ -1189,9 +1220,9 @@ class Kronolith_Event {
     }
 
     /**
-     * Retrieve the category this event belongs to.
+     * Retrieve the category of this event.
      *
-     * @return integer  The identifier of the category this event belongs to.
+     * @return string  The category of this event.
      */
     function getCategory()
     {
@@ -1258,7 +1289,7 @@ class Kronolith_Event {
      */
     function setAttendees($attendees)
     {
-        $this->attendees = $attendees;
+        $this->attendees = array_change_key_case($attendees);
     }
 
     /**
@@ -1271,6 +1302,7 @@ class Kronolith_Event {
      */
     function addAttendee($email, $attendance, $response)
     {
+        $email = String::lower($email);
         if ($attendance == KRONOLITH_PART_IGNORE) {
             if (isset($this->attendees[$email])) {
                 $attendance = $this->attendees[$email]['attendance'];
@@ -1292,6 +1324,7 @@ class Kronolith_Event {
      */
     function removeAttendee($email)
     {
+        $email = String::lower($email);
         if (isset($this->attendees[$email])) {
             unset($this->attendees[$email]);
         }
@@ -1318,6 +1351,7 @@ class Kronolith_Event {
      */
     function hasAttendee($email)
     {
+        $email = String::lower($email);
         return isset($this->attendees[$email]);
     }
 
@@ -1997,7 +2031,7 @@ class Kronolith_Event {
         $after = &new Horde_Date($afterDate);
         $after->correct();
 
-        if ($this->start->compareDate($after) >= 0) {
+        if ($this->start->compareDateTime($after) >= 0) {
             return new Horde_Date($this->start);
         }
 
@@ -2005,21 +2039,15 @@ class Kronolith_Event {
             return false;
         }
 
-        if ($this->hasRecurEnd()) {
-            $this->recurEnd->hour = 23;
-            $this->recurEnd->min = 59;
-            $this->recurEnd->sec = 59;
-        }
-
         switch ($this->getRecurType()) {
         case KRONOLITH_RECUR_DAILY:
             $diff = Date_Calc::dateDiff($this->start->mday, $this->start->month, $this->start->year, $after->mday, $after->month, $after->year);
             $recur = ceil($diff / $this->recurInterval) * $this->recurInterval;
-            $next = $this->start;
+            $next = Util::cloneObject($this->start);
             list($next->mday, $next->month, $next->year) = explode('/', Date_Calc::daysToDate(Date_Calc::dateToDays($next->mday, $next->month, $next->year) + $recur, '%e/%m/%Y'));
             if ((!$this->hasRecurEnd() ||
-                 $next->compareDate($this->recurEnd) <= 0) &&
-                $next->compareDate($after) >= 0) {
+                 $next->compareDateTime($this->recurEnd) <= 0) &&
+                $next->compareDateTime($after) >= 0) {
                 return new Horde_Date($next);
             }
             break;
@@ -2044,22 +2072,24 @@ class Kronolith_Event {
             $next = $start_week;
             list($next->mday, $next->month, $next->year) = explode('/', Date_Calc::daysToDate(Date_Calc::dateToDays($next->mday, $next->month, $next->year) + $recur, '%e/%m/%Y'));
             $next = &new Horde_Date($next);
-            while ($next->compareDate($after) < 0 && $next->compareDate($after_week_end) < 0) {
+            while ($next->compareDateTime($after) < 0 &&
+                   $next->compareDateTime($after_week_end) < 0) {
                 ++$next->mday;
                 $next->correct();
             }
             if (!$this->hasRecurEnd() ||
-                $next->compareDate($this->recurEnd) <= 0) {
-                if ($next->compareDate($after_week_end) >= 0) {
+                $next->compareDateTime($this->recurEnd) <= 0) {
+                if ($next->compareDateTime($after_week_end) >= 0) {
                     return $this->nextRecurrence($after_week_end);
                 }
-                while (!$this->recurOnDay((int)pow(2, $next->dayOfWeek())) && $next->compareDate($after_week_end) < 0) {
+                while (!$this->recurOnDay((int)pow(2, $next->dayOfWeek())) &&
+                       $next->compareDateTime($after_week_end) < 0) {
                     ++$next->mday;
                     $next->correct();
                 }
                 if (!$this->hasRecurEnd() ||
-                    $next->compareDate($this->recurEnd) <= 0) {
-                    if ($next->compareDate($after_week_end) >= 0) {
+                    $next->compareDateTime($this->recurEnd) <= 0) {
+                    if ($next->compareDateTime($after_week_end) >= 0) {
                         return $this->nextRecurrence($after_week_end);
                     } else {
                         return $next;
@@ -2070,7 +2100,7 @@ class Kronolith_Event {
 
         case KRONOLITH_RECUR_DAY_OF_MONTH:
             $start = Util::cloneObject(new Horde_Date($this->start));
-            if ($after->compareDate($start) < 0) {
+            if ($after->compareDateTime($start) < 0) {
                 $after = $start;
             }
 
@@ -2111,7 +2141,7 @@ class Kronolith_Event {
 
                 // Bail if we've gone past the end of recurrence.
                 if ($this->hasRecurEnd() &&
-                    $this->recurEnd->compareDate($start) < 0) {
+                    $this->recurEnd->compareDateTime($start) < 0) {
                     return false;
                 }
             } while (true);
@@ -2140,13 +2170,13 @@ class Kronolith_Event {
                 $next = &Util::cloneObject($estart);
                 $next->setNthWeekday($weekday, $nth);
 
-                if ($next->compareDate($after) < 0) {
+                if ($next->compareDateTime($after) < 0) {
                     // We haven't made it past $after yet, try
                     // again.
                     continue;
                 }
                 if ($this->hasRecurEnd() &&
-                    $next->compareDate($this->recurEnd) > 0) {
+                    $next->compareDateTime($this->recurEnd) > 0) {
                     // We've gone past the end of recurrence; we can
                     // give up now.
                     return false;
@@ -2181,7 +2211,7 @@ class Kronolith_Event {
 
             // We've gone past the end of recurrence; give up.
             if ($this->hasRecurEnd() &&
-                $this->recurEnd->compareDate($estart) < 0) {
+                $this->recurEnd->compareDateTime($estart) < 0) {
                 return false;
             }
 
